;;; The contents of this directory structure is subject to the following
;;; copyright notice:
;;;
;;;	Copyright (c) 2006, 2007 Andrew A. Berlin
;;;
;;;	This program is free software; you can redistribute it and/or
;;;	modify it under the terms of the GNU General Public License as
;;;	published by the Free Software Foundation; either version 2 of
;;;	the License, or (at your option) any later version.
;;;
;;;	This program is distributed in the hope that it will be
;;;	useful, but WITHOUT ANY WARRANTY; without even the implied
;;;	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
;;;	PURPOSE.  See the GNU General Public License for more details.
;;;
;;;	You should have received a copy of the GNU General Public
;;;	License along with this program; if not, write to the Free
;;;	Software Foundation, Inc., 59 Temple Place - Suite 330,
;;;	Boston, MA 02111-1307, USA.

(define actuator-x-offset)
(define actuator-z-offset)
(define target-x-offset)
(define target-z-offset)

(define *char-max-sideaccel* (* 10 9.8)) ;;10g max sidewall 
(define *char-min-prop-accel* (* 1 9.8)) ;;1g min propulsive
(define *char-max-prop-accel* (* 5 9.8)) ;;5g max propulsive

(set! actuator-x-offset 0)
(set! actuator-z-offset 0)
(set! target-x-offset 0)
(set! target-z-offset 0)

(define *sidewall-constraint-active* #t)   ;;modified only by without-sideaccel-constraint

(define (without-sideaccel-constraint p)
  (let ((orig-side-accel))
    (dynamic-wind (lambda ()
		    (set! *sidewall-constraint-active* false)
		    (set! orig-side-accel
			  (set! *char-max-sideaccel* 1e10)))
		  p
		  (lambda ()
		    (set! *sidewall-constraint-active* true)
		    (set! *char-max-sideaccel* orig-side-accel)))))

(define (characterize-worlds worlds #!optional print-just-operating-region max-sideaccel min-prop-accel max-prop-accel)
  (if (default-object? max-sideaccel)
      (set! max-sideaccel *char-max-sideaccel*))
  (if (default-object? min-prop-accel)
      (set! min-prop-accel *char-min-prop-accel*))
  (if (default-object? max-prop-accel)
      (set! max-prop-accel *char-max-prop-accel*))
  (map (lambda (w)
	 (characterize-world w max-sideaccel min-prop-accel max-prop-accel print-just-operating-region))
       worlds))

(define-structure world-characterization world operating-min operating-max operating-size)

#|
Have a region that is initially the entire fallopian tube 0 to L
Next step is to limit by sideaccel < max-sideaccel
In event there is an alpha at which alpha=max-sideaccel, then:
Reduce region max to that alpha
In the event there is NOT an alpha at which alpha=max-sideaccel, then: check whether sideaccel is Ok at L
If sideaccel is Ok at L, then sideaccel does not impose any constraints.
If sideaccel is not Ok at L, then there is no operating region.

		 
|#

(define-structure operating-region min max comments)

(define (limit-operating-region-based-on-sideaccel! operating-region world max-sideaccel)
  (if (or (not (number? (operating-region-min operating-region)))
	  (not (number? (operating-region-max operating-region))))
      'no-region
      (let ((opt-result (compute-sidewall-accel-balance-pt world max-sideaccel)))
	(if (seek-target-converged? opt-result)

	    ;;THERE IS AN OPTIMIZATION-BASED LIMIT
	    (limit-region-max! operating-region
			       (car opt-result)
			       "Sidewall Max")

	    ;;THERE IS NOT AN OPTIMIZATION-IMPOSED LIMIT --> DID NOT CONVERGE --> SAME FLAT VALUE EVERYWHERE
	    (begin
	      (let ((f-at-min (compute-sidewall-accel world
						      (operating-region-min operating-region)))
		    (f-at-max (compute-sidewall-accel world
						      (operating-region-max operating-region))))
		;;sanity check
;;		(region-sanity-check f-at-min f-at-max "limit-operating-region-based-on-sideaccel!: ")

		;;CHECK WHETHER THE VALUE THAT IS IN PLAY IS ACCEPTABLE
		(if (<= f-at-min max-sideaccel)
		    ;;ALL IS OK 
		    (set-operating-region-comments! operating-region
						    (cons "Region max (no opt.) NOT limited by Sidewall Max"
							  (operating-region-comments operating-region)))
		    ;;ALL IS NOT OK - NO REGION
		    (let ((msg (format #f "NO REGION - ~a optimization failed at value ~a"
				       "Sidewall Max"
				       f-at-min)))
		      (set-operating-region-min! operating-region msg)
		      (set-operating-region-max! operating-region msg))
		    )))
	    ))))

(define (limit-operating-region-based-on-max-prop-accel! operating-region world max-prop-accel)
  (if (or (not (number? (operating-region-min operating-region)))
	  (not (number? (operating-region-max operating-region))))
      'no-region
      (let ((opt-result (compute-prop-accel-balance-pt world max-prop-accel)))
	(if (seek-target-converged? opt-result)
	    
	    ;;THERE IS AN OPTIMIZATION-BASED LIMIT
	    (limit-region-max! operating-region
			       (car opt-result) 
			       "Prop Max")
	    
	    ;;THERE IS NOT AN OPTIMIZATION-IMPOSED LIMIT --> DID NOT CONVERGE --> SAME FLAT VALUE EVERYWHERE
	    (begin
	      (let ((f-at-min (compute-prop-accel world
						  (operating-region-min operating-region)))
		    (f-at-max (compute-prop-accel world
						  (operating-region-max operating-region))))
		;;sanity check
;;		(region-sanity-check f-at-min f-at-max "limit-operating-region-based-on-max-prop-accel!: ")

		;;CHECK WHETHER THE VALUE THAT IS IN PLAY IS ACCEPTABLE
		(if (<= f-at-min max-prop-accel)
		    ;;ALL IS OK 
		    (set-operating-region-comments! operating-region
						    (cons "Region max (no opt.) NOT limited by Prop Accel Max"
							  (operating-region-comments operating-region)))
		    ;;ALL IS NOT OK - NO REGION
		    (let ((msg (format #f "NO REGION - ~a optimization failed at value ~a"
				       "Prop Accel Max"
				       f-at-min)))
		      (set-operating-region-min! operating-region msg)
		      (set-operating-region-max! operating-region msg))
		    )))
	    ))))

(define (limit-operating-region-based-on-min-prop-accel! operating-region world min-prop-accel)
  (if (or (not (number? (operating-region-min operating-region)))
	  (not (number? (operating-region-max operating-region))))
      'no-region
      (let ((opt-result (compute-prop-accel-balance-pt world min-prop-accel)))
	(if (seek-target-converged? opt-result)
	    
	    ;;THERE IS AN OPTIMIZATION-BASED LIMIT
	    (limit-region-min! operating-region
			       (car opt-result) 
			       "Prop Min")
	    
	    ;;THERE IS NOT AN OPTIMIZATION-IMPOSED LIMIT --> DID NOT CONVERGE --> SAME FLAT VALUE EVERYWHERE
	    (begin
	      (let ((f-at-min (compute-prop-accel world
						  (operating-region-min operating-region)))
		    (f-at-max (compute-prop-accel world
						  (operating-region-max operating-region))))
		;;sanity check
;;		(region-sanity-check f-at-min f-at-max "limit-operating-region-based-on-min-prop-accel!: ")

		;;CHECK WHETHER THE VALUE THAT IS IN PLAY IS ACCEPTABLE
		(if (>= f-at-min min-prop-accel)
		    ;;ALL IS OK 
		    (set-operating-region-comments! operating-region
						    (cons "Region min (no opt.) NOT limited by Prop Accel Min"
							  (operating-region-comments operating-region)))
		    ;;ALL IS NOT OK - NO REGION
		    (let ((msg (format #f "NO REGION - ~a optimization failed at value ~a"
				       "Prop Accel Min"
				       f-at-min)))
		      (set-operating-region-min! operating-region msg)
		      (set-operating-region-max! operating-region msg))
		    )))
	    ))))

;;sanity check is overly conservative.
;;could also be a situation where x at which f(x) = target simply lies outside bounds of 0 to L
(define (region-sanity-check f-at-min f-at-max textnote)
  (if (not (= f-at-min f-at-max))
      (error
       (format #f "~a Optimization did not converge but different value at region max and min detected"
	       textnote))))

(define (limit-region-max! operating-region newval textnote)
  (if (< newval (operating-region-max operating-region))
      ;;LIMIT SHRINKS REGION
      (begin
	(set-operating-region-max! operating-region 
				   newval)
	(set-operating-region-comments! operating-region
					(cons (format #f "Max limited by ~a to ~a"
						      textnote
						      newval)
					      (operating-region-comments operating-region)))
	(if (< (operating-region-max operating-region) ;;is new max < min?
	       (operating-region-min operating-region))
	    (let ((msg (format #f "NO Region: Region anihilated by stringent max limit by ~a" textnote)))
	      (set-operating-region-min! operating-region msg)
	      (set-operating-region-comments! operating-region
					      (cons msg (operating-region-comments operating-region)))))
	) ;;end of shrink region
      ;;LIMIT DOES NOT SHRINK REGION
      (begin 
	(set-operating-region-comments! operating-region
					(cons (format #f "Region Max NOT limited by ~a"
						      textnote)
					      (operating-region-comments operating-region)))
	)))

(define (limit-region-min! operating-region newval textnote)
  (if (> newval (operating-region-min operating-region))
      ;;LIMIT SHRINKS REGION
      (begin 
	(set-operating-region-min! operating-region 
				   newval)
	(set-operating-region-comments! operating-region
					(cons (format #f "Min limited by ~a to ~a"
						      textnote
						      newval)
					      (operating-region-comments operating-region)))

	(if (> (operating-region-min operating-region) ;;is new min > max ?
	       (operating-region-max operating-region))
	    (let ((msg (format #f "NO Region: Region anihilated by stringent min limit by ~a" textnote)))
	      (set-operating-region-max! operating-region msg)
	      (set-operating-region-comments! operating-region
					      (cons msg (operating-region-comments operating-region)))))
	) ;;end of shrink region
      ;;LIMIT DOES NOT SHRINK REGION
      (begin 
	(set-operating-region-comments! operating-region
					(cons (format #f "Region Min NOT limited by ~a"
						      textnote)
					      (operating-region-comments operating-region)))
	)))

(define (characterize-world world #!optional
			    max-sideaccel min-prop-accel max-prop-accel
			    print-just-operating-region quiet-mode)
  (if (default-object? max-sideaccel)
      (set! max-sideaccel *char-max-sideaccel*))
  (if (default-object? min-prop-accel)
      (set! min-prop-accel *char-min-prop-accel*))
  (if (default-object? max-prop-accel)
      (set! max-prop-accel *char-max-prop-accel*))
  (if (default-object? print-just-operating-region)
      (set! print-just-operating-region #f))
  (if (default-object? quiet-mode)
      (set! quiet-mode #f))
  (let* ((topology (world-topology world))
	 (L (topology-L topology))
	 (region (make-operating-region 0 L '())))

    (limit-operating-region-based-on-sideaccel! region world max-sideaccel)
    (limit-operating-region-based-on-max-prop-accel! region world
						     max-prop-accel)
    (limit-operating-region-based-on-min-prop-accel! region world
						     min-prop-accel)

    (let ((operating-min (operating-region-min region))
	  (operating-max (operating-region-max region)))
      (if (and (not print-just-operating-region) (not quiet-mode))
	  (begin
	    (format #t "~%-------------------------------------------------------------------------------")
	    (format #t "~%World ~a" (world-name world))
	    (describe-world world)
	    (format #t "~%Computing Minimum Prop Accel Balance")
	    (report-prop-accel-balance world min-prop-accel)
	    (format #t "~%Computing Maximum Prop Accel Balance")
	    (report-prop-accel-balance world max-prop-accel)
	    (format #t "~%Computing Sidewall accel limit")
	    (report-sidewall-accel-balance world max-sideaccel)
	    ))
      (if (not quiet-mode)
	  (begin 
	    (format #t "~%World ~a effective operating region: ~a to ~a size: ~a cm"
		    (world-name world)
		    operating-min
		    operating-max
		    (* 100 (- operating-max
			      operating-min))
		    )
	    (newline)
	    (pp (operating-region-comments region))
	    (newline)))

      (make-world-characterization world
				   operating-min
				   operating-max
				   (if (and (number? operating-max)
					    (number? operating-min))
				       (- operating-max operating-min)
				       "Empty Region")
				   )
      )))




(define testtube
  (make-fallopian-tube 'testtube-fallopian-tube
		       (make-3point 0 0 0) ;;startpt
		       (make-3point 0 (in->m 4.5) 0) ;;endpt
		       tube-3d-position
		       tube-3d-orientation
		       (in->m 4.5) ;;tube length
		       ))


(define testtube-world-3
  (define-world 'testtube-levitation-3
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-1mmx3mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point target-x-offset
							 0
							 target-z-offset)
					    (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

(define testtube-world-4
  (define-world 'testtube-levitation-4
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-Gaussboys-1mmx4mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
					    (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

(define testtube-world-7
  (define-world 'testtube-levitation-7
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-1mmx7mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
					    (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

(define testtube-world-10
  (define-world 'testtube-levitation-10
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-1mmx10mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
					    (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

(define testtube-world-20
  (define-world 'testtube-levitation-20
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-1mmx20mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
				  (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

(define testtube-world-4-dia2
  (define-world 'testtube-levitation-4-dia2
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-2mmx4mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
					    (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))


(set! actuator-x-offset (cm->m 3))
(set! actuator-z-offset 0)
(set! target-x-offset (cm->m 3))
(set! target-z-offset 0)


(define testtube-world-3-offset-3
  (define-world 'testtube-levitation-3-offset-3
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-1mmx3mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
					    (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

(define testtube-world-4-offset-3
  (define-world 'testtube-levitation-4-offset-3
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-Gaussboys-1mmx4mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
					    (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

(define testtube-world-7-offset-3
  (define-world 'testtube-levitation-7-offset-3
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-1mmx7mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
					    (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

(define testtube-world-10-offset-3
  (define-world 'testtube-levitation-10-offset-3
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-1mmx10mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
					    (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

(define testtube-world-20-offset-3
  (define-world 'testtube-levitation-20-offset-3
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-1mmx20mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
					    (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

(define testtube-world-4-dia2-offset-3
  (define-world 'testtube-levitation-4-dia2-offset-3
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-2mmx4mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
					    (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))



;;;laterally offset AND angled back towards origin of tube

(set! actuator-x-offset (cm->m 3))
(set! actuator-z-offset 0)
(set! target-x-offset 0)
(set! target-z-offset 0)

(define testtube-world-3-offset-3-angle-0
  (define-world 'testtube-levitation-3-offset-3-angle-0
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-1mmx3mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
					    (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

(define testtube-world-4-offset-3-angle-0
  (define-world 'testtube-levitation-4-offset-3-angle-0
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-Gaussboys-1mmx4mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
					    (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

(define testtube-world-7-offset-3-angle-0
  (define-world 'testtube-levitation-7-offset-3-angle-0
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-1mmx7mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
					    (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

(define testtube-world-10-offset-3-angle-0
  (define-world 'testtube-levitation-10-offset-3-angle-0
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-1mmx10mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
					    (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

(define testtube-world-20-offset-3-angle-0
  (define-world 'testtube-levitation-20-offset-3-angle-0
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-1mmx20mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
				  (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

(define testtube-world-4-dia2-offset-3-angle-0
  (define-world 'testtube-levitation-4-dia2-offset-3-angle-0
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-2mmx4mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
					    (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))



;;;laterally offset AND angled away from origin of tube by twice the offset

(set! actuator-x-offset (cm->m 3))
(set! actuator-z-offset 0)
(set! target-x-offset (* 2 actuator-x-offset))
(set! target-z-offset 0)


(define testtube-world-3-offset-3-angle-twice-offset
  (define-world 'testtube-levitation-3-offset-3-angle-twice-offset
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-1mmx3mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
					    (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

(define testtube-world-4-offset-3-angle-twice-offset
  (define-world 'testtube-levitation-4-offset-3-angle-twice-offset
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-Gaussboys-1mmx4mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
					    (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

(define testtube-world-7-offset-3-angle-twice-offset
  (define-world 'testtube-levitation-7-offset-3-angle-twice-offset
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-1mmx7mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
					    (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

(define testtube-world-10-offset-3-angle-twice-offset
  (define-world 'testtube-levitation-10-offset-3-angle-twice-offset
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-1mmx10mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
					    (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

(define testtube-world-20-offset-3-angle-twice-offset
  (define-world 'testtube-levitation-20-offset-3-angle-twice-offset
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-1mmx20mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
				  (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

(define testtube-world-4-dia2-offset-3-angle-twice-offset
  (define-world 'testtube-levitation-4-dia2-offset-3-angle-twice-offset
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-2mmx4mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
					    (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

;;;laterally offset AND angled back towards alphamin of on-axis actuation
;;;which is at 0.0586cm

(set! actuator-x-offset (cm->m 3))
(set! actuator-z-offset 0)
(set! target-x-offset 0.0586) ;;alphamin of on-axis actuation
(set! target-z-offset 0)

(define testtube-world-3-offset-3-angle-towards-alphamin
  (define-world 'testtube-levitation-3-offset-3-angle-0-towards-alphamin
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-1mmx3mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
					    (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

(define testtube-world-4-offset-3-angle-towards-alphamin
  (define-world 'testtube-levitation-4-offset-3-angle-towards-alphamin
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-Gaussboys-1mmx4mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
					    (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

(define testtube-world-7-offset-3-angle-towards-alphamin
  (define-world 'testtube-levitation-7-offset-3-angle-towards-alphamin
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-1mmx7mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
					    (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

(define testtube-world-10-offset-3-angle-towards-alphamin
  (define-world 'testtube-levitation-10-offset-3-angle-towards-alphamin
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-1mmx10mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
					    (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

(define testtube-world-20-offset-3-angle-towards-alphamin
  (define-world 'testtube-levitation-20-offset-3-angle-towards-alphamin
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-1mmx20mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
				  (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

(define testtube-world-4-dia2-offset-3-angle-towards-alphamin
  (define-world 'testtube-levitation-4-dia2-offset-3-angle-towards-alphamin
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-2mmx4mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
					    (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))




;;;laterally offset AND angled back towards alphamax of on-axis actuation
;;;which is at 0.0975

(set! actuator-x-offset (cm->m 3))
(set! actuator-z-offset 0)
(set! target-x-offset 0.0975) ;;alphamax of on-axis actuation
(set! target-z-offset 0)

(define testtube-world-3-offset-3-angle-towards-alphamax
  (define-world 'testtube-levitation-3-offset-3-angle-towards-alphamax
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-1mmx3mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
					    (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

(define testtube-world-4-offset-3-angle-towards-alphamax
  (define-world 'testtube-levitation-4-offset-3-angle-towards-alphamax
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-Gaussboys-1mmx4mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
					    (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

(define testtube-world-7-offset-3-angle-towards-alphamax
  (define-world 'testtube-levitation-7-offset-3-angle-towards-alphamax
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-1mmx7mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
					    (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

(define testtube-world-10-offset-3-angle-towards-alphamax
  (define-world 'testtube-levitation-10-offset-3-angle-towards-alphamax
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-1mmx10mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
					    (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

(define testtube-world-20-offset-3-angle-towards-alphamax
  (define-world 'testtube-levitation-20-offset-3-angle-towards-alphamax
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-1mmx20mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
				  (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

(define testtube-world-4-dia2-offset-3-angle-towards-alphamax
  (define-world 'testtube-levitation-4-dia2-offset-3-angle-towards-alphamax
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-2mmx4mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (3point-+ (make-3point
					     target-x-offset
					     0
					     target-z-offset)
					    (topology-startpt simple-linear-fallopian-tube)))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))




(characterize-worlds
 (append (list testtube-world-3
	       testtube-world-4
	       testtube-world-7
	       testtube-world-10
	       testtube-world-20
	       testtube-world-4-dia2)
	 (list testtube-world-3-offset-3
	       testtube-world-4-offset-3
	       testtube-world-7-offset-3
	       testtube-world-10-offset-3
	       testtube-world-20-offset-3
	       testtube-world-4-dia2-offset-3)
	 (list testtube-world-3-offset-3-angle-0
	       testtube-world-4-offset-3-angle-0
	       testtube-world-7-offset-3-angle-0
	       testtube-world-10-offset-3-angle-0
	       testtube-world-20-offset-3-angle-0
	       testtube-world-4-dia2-offset-3-angle-0)))

(characterize-worlds
 (append (list testtube-world-3
	       testtube-world-4
	       testtube-world-7
	       testtube-world-10
	       testtube-world-20
	       testtube-world-4-dia2)
	 (list testtube-world-3-offset-3
	       testtube-world-4-offset-3
	       testtube-world-7-offset-3
	       testtube-world-10-offset-3
	       testtube-world-20-offset-3
	       testtube-world-4-dia2-offset-3)
	 (list testtube-world-3-offset-3-angle-0
	       testtube-world-4-offset-3-angle-0
	       testtube-world-7-offset-3-angle-0
	       testtube-world-10-offset-3-angle-0
	       testtube-world-20-offset-3-angle-0
	       testtube-world-4-dia2-offset-3-angle-0)))

(newline)
(characterize-worlds 
 (list testtube-world-3
       testtube-world-3-offset-3-angle-0
       testtube-world-3-offset-3
       testtube-world-3-offset-3-angle-towards-alphamin
;       testtube-world-3-offset-3-angle-towards-alphamax
       testtube-world-3-offset-3-angle-twice-offset
       )
 'just-operating-region)
(newline)
(characterize-worlds 
 (list testtube-world-4
       testtube-world-4-offset-3-angle-0
       testtube-world-4-offset-3
       testtube-world-4-offset-3-angle-towards-alphamin
;       testtube-world-4-offset-3-angle-towards-alphamax
       testtube-world-4-offset-3-angle-twice-offset
       )
 'just-operating-region)
(newline)
(characterize-worlds 
 (list testtube-world-7
       testtube-world-7-offset-3-angle-0
       testtube-world-7-offset-3
       testtube-world-7-offset-3-angle-towards-alphamin
;       testtube-world-7-offset-3-angle-towards-alphamax
       testtube-world-7-offset-3-angle-twice-offset
       )
       
 'just-operating-region)
(newline)
(characterize-worlds 
 (list testtube-world-10
       testtube-world-10-offset-3-angle-0
       testtube-world-10-offset-3
       testtube-world-10-offset-3-angle-towards-alphamin
;       testtube-world-10-offset-3-angle-towards-alphamax
       testtube-world-10-offset-3-angle-twice-offset
)
 'just-operating-region)
(newline)
(characterize-worlds 
 (list testtube-world-20
       testtube-world-20-offset-3-angle-0
       testtube-world-20-offset-3
       testtube-world-20-offset-3-angle-towards-alphamin
;       testtube-world-20-offset-3-angle-towards-alphamax
       testtube-world-20-offset-3-angle-twice-offset
       )
 'just-operating-region)
(newline)
(characterize-worlds 
 (list testtube-world-4-dia2
       testtube-world-4-dia2-offset-3-angle-0
       testtube-world-4-dia2-offset-3
       testtube-world-4-dia2-offset-3-angle-towards-alphamin
;       testtube-world-4-dia2-offset-3-angle-towards-alphamax
       testtube-world-20-offset-3-angle-twice-offset
       )
 'just-operating-region)

(define *optimize-min-location-offset* (cm->m 0))
(define *optimize-max-location-offset* (cm->m 6.))
(define *optimize-default-min-target-x-offset* (cm->m -50.))
(define *optimize-default-max-target-x-offset* (cm->m 10.))



;;Experiments

;;lev-distance is distance from actuator center at operating min to probe center
;;also used by expt2
(define (expt1-compute-lev-distance wc)
  (let ((opmin (world-characterization-operating-min wc)))
    (if (number? opmin)
	(3point-distance (actuator-location (car (world-actuators (world-characterization-world wc))))
			 (compute-probe-3pt-position (world-characterization-world wc) (world-characterization-operating-min wc)))
	*noplot*)))

(define (expt1-lev-distance worlds)
  (without-sideaccel-constraint
   (lambda ()
     (let* ((characterizations (characterize-worlds worlds)))
       (map (lambda (wc) (list (list 'world-name (world-name (world-characterization-world wc)))
			       (list 'operating-distance-cm
				     (* (m->cm (expt1-compute-lev-distance wc))))))
	    characterizations)))))

(define (expt2-compute-lev-distance-on-axis wc)
  (let ((opmin (world-characterization-operating-min wc)))
    (if (number? opmin)
	(- (3point-y (actuator-location (car (world-actuators (world-characterization-world wc)))))
	   (3point-y (compute-probe-3pt-position (world-characterization-world wc) opmin)))
	*noplot*)))

(define (expt1-region-size worlds)
  (let* ((characterizations (characterize-worlds worlds)))
    (map (lambda (wc) (list (list 'world-name (world-name (world-characterization-world wc)))
			    (list 'region-size-cm
				  (* (m->cm (world-characterization-operating-size wc))))))
	 characterizations)))

(with-output-to-file "expt1-lev-distance.log"
  (lambda ()
    (pp
     (expt1-lev-distance
      (list testtube-world-3
	    testtube-world-4
	    testtube-world-7
	    testtube-world-10
	    testtube-world-20
	    testtube-world-4-dia2)))))

(with-output-to-file "expt1-region-size.log"
  (lambda ()
    (pp
     (expt1-region-size
      (list testtube-world-3
	    testtube-world-4
	    testtube-world-7
	    testtube-world-10
	    testtube-world-20
	    testtube-world-4-dia2)))))

#|

;;Experiment 2

;;TESTTUBE-WORLD-4
;;operating-region-size
(without-sideaccel-constraint
 (lambda ()
   (plot-actuator-location-offset-vs-characterization-variable-no-rotation
    testtube-world-4
    (named-lambda (operating-region-size-cm-nosideaccel wc)
      (let ((size (world-characterization-operating-size wc)))
	(if (number? size)
	    (m->cm size)
	    *noplot*)))
    "expt2-data")))

;;operating-region-size
(plot-actuator-location-offset-vs-characterization-variable-no-rotation
 testtube-world-4
 (named-lambda (operating-region-size-cm wc)
   (let ((size (world-characterization-operating-size wc)))
     (if (number? size)
	 (m->cm size)
	 *noplot*)))
 "expt2-data")

;;operating-max
(plot-actuator-location-offset-vs-characterization-variable-no-rotation
 testtube-world-4
 (named-lambda (operating-max-cm wc)
   (let ((num (world-characterization-operating-max wc)))
     (if (number? num)
	 (m->cm num)
	 *noplot*)))
 "expt2-data")

(without-sideaccel-constraint
 (lambda ()
   (plot-actuator-location-offset-vs-characterization-variable-no-rotation
    testtube-world-4
    (named-lambda (operating-max-cm wc)
      (let ((num (world-characterization-operating-max wc)))
	(if (number? num)
	    (m->cm num)
	    *noplot*)))
    "expt2-data")))

;;operating-min
(plot-actuator-location-offset-vs-characterization-variable-no-rotation
 testtube-world-4
 (named-lambda (operating-min-cm wc)
   (let ((num (world-characterization-operating-min wc)))
     (if (number? num)
	 (m->cm num)
	 *noplot*)))
 "expt2-data")

;;working-distance-on-axis
(plot-actuator-location-offset-vs-characterization-variable-no-rotation
 testtube-world-4
 (named-lambda (working-distance-on-axis-cm x)
   (let ((num (expt2-compute-lev-distance-on-axis x)))
     (if (number? num)
	 (* 100 num)
	 *noplot*)))
 "expt2-data")

;;working-distance-absolute
(plot-actuator-location-offset-vs-characterization-variable-no-rotation
 testtube-world-4
 (named-lambda (working-distance-absolute-cm x)
   (let ((num (expt1-compute-lev-distance x)))
     (if (number? num)
	 (* 100 num)
	 *noplot*)))
 "expt2-data")



;;TESTTUBE-WORLD-10
;;operating-region-size
(without-sideaccel-constraint
 (lambda ()
   (plot-actuator-location-offset-vs-characterization-variable-no-rotation
    testtube-world-10
    (named-lambda (operating-region-size-cm-nosideaccel wc)
      (let ((size (world-characterization-operating-size wc)))
	(if (number? size)
	    (m->cm size)
	    *noplot*)))
    "expt2-data")))

;;operating-region-size
(plot-actuator-location-offset-vs-characterization-variable-no-rotation
 testtube-world-10
 (named-lambda (operating-region-size-cm wc)
   (let ((size (world-characterization-operating-size wc)))
     (if (number? size)
	 (m->cm size)
	 *noplot*)))
 "expt2-data")

;;operating-max
(plot-actuator-location-offset-vs-characterization-variable-no-rotation
 testtube-world-10
 (named-lambda (operating-max-cm wc)
   (let ((num (world-characterization-operating-max wc)))
     (if (number? num)
	 (m->cm num)
	 *noplot*)))
 "expt2-data")

;;operating-min
(plot-actuator-location-offset-vs-characterization-variable-no-rotation
 testtube-world-10
 (named-lambda (operating-min-cm wc)
   (let ((num (world-characterization-operating-min wc)))
     (if (number? num)
	 (m->cm num)
	 *noplot*)))
 "expt2-data")

;;working-distance-on-axis
(plot-actuator-location-offset-vs-characterization-variable-no-rotation
 testtube-world-10
 (named-lambda (working-distance-on-axis-cm x)
   (let ((num (expt2-compute-lev-distance-on-axis x)))
     (if (number? num)
	 (* 100 num)
	 *noplot*)))
 "expt2-data")

;;working-distance-absolute
(plot-actuator-location-offset-vs-characterization-variable-no-rotation
 testtube-world-10
 (named-lambda (working-distance-absolute-cm x)
   (let ((num (expt1-compute-lev-distance x)))
     (if (number? num)
	 (* 100 num)
	 *noplot*)))
 "expt2-data")


;;Experiment 3
;;effect of rotation on working region size
(plot-actuator-target-offset-vs-characterization-variable
 testtube-world-4
 (named-lambda (working-region-size-cm x)
   (let ((r (world-characterization-operating-size x)))
     (if (number? r)
	 (* r 100)
	 *noplot*)))
 0)

(plot-actuator-target-offset-vs-characterization-variable
 testtube-world-4
 (named-lambda (working-region-size-cm x)
   (let ((r (world-characterization-operating-size x)))
     (if (number? r)
	 (* r 100)
	 *noplot*)))
 .01
 'relative
 )

(without-sideaccel-constraint
 (lambda ()
   (plot-actuator-target-offset-vs-characterization-variable
    testtube-world-4
    (named-lambda (working-region-size-cm x)
      (let ((r (world-characterization-operating-size x)))
	(if (number? r)
	    (* r 100)
	    *noplot*)))
    .01
    'relative
    )))

(plot-actuator-target-offset-vs-characterization-variable
 testtube-world-4
 (named-lambda (working-region-size-cm x)
   (let ((r (world-characterization-operating-size x)))
     (if (number? r)
	 (* r 100)
	 *noplot*)))
 .03
 'relative)

(plot-actuator-target-offset-vs-characterization-variable
 testtube-world-4
 (named-lambda (working-distance-on-axis-cm x)
   (let ((r (expt2-compute-lev-distance-on-axis x)))
     (if (number? r)
	 (* r 100)
	 *noplot*)))
 0.
 'relative)

(plot-actuator-target-offset-vs-characterization-variable
 testtube-world-4
 (named-lambda (working-distance-on-axis-cm x)
   (let ((r (expt2-compute-lev-distance-on-axis x)))
     (if (number? r)
	 (* r 100)
	 *noplot*)))
 0.01
 'relative)

(plot-actuator-target-offset-vs-characterization-variable
 testtube-world-4
 (named-lambda (working-distance-on-axis-cm x)
   (let ((r (expt2-compute-lev-distance-on-axis x)))
     (if (number? r)
	 (* r 100)
	 *noplot*)))
 .03
 'relative)


;;Maximization of region size

(plot-actuator-location-offset-vs-optimal-target-offset testtube-world-4)
(plot-actuator-location-offset-vs-RELATIVE-target-offset testtube-world-4)

(without-sideaccel-constraint
 (lambda ()
   (plot-actuator-location-offset-vs-optimal-target-offset testtube-world-4)))
(without-sideaccel-constraint
 (lambda ()
   (plot-actuator-location-offset-vs-RELATIVE-target-offset testtube-world-4)))

;;Experiment 4
;;Operating region size and working distance as a function of of lateral offset, with automatic rotation to optimize working distance

;;working-distance-on-axis
(plot-actuator-location-offset-vs-characterization-variable
 testtube-world-4
 (named-lambda (working-distance-on-axis-cm x)
   (let ((num (expt2-compute-lev-distance-on-axis x)))
     (if (number? num)
	 (* 100 num)
	 *noplot*)))
 "expt2-data")

;;operating-region-size
(plot-actuator-location-offset-vs-characterization-variable
 testtube-world-4
 (named-lambda (operating-region-size-cm wc)
   (let ((size (world-characterization-operating-size wc)))
     (if (number? size)
	 (m->cm size)
	 *noplot*)))
 "expt2-data")


|#


