;;; The contents of this directory structure is subject to the following
;;; copyright notice:
;;;
;;;	Copyright (c) 2006, 2007 Andrew A. Berlin
;;;
;;;	This program is free software; you can redistribute it and/or
;;;	modify it under the terms of the GNU General Public License as
;;;	published by the Free Software Foundation; either version 2 of
;;;	the License, or (at your option) any later version.
;;;
;;;	This program is distributed in the hope that it will be
;;;	useful, but WITHOUT ANY WARRANTY; without even the implied
;;;	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
;;;	PURPOSE.  See the GNU General Public License for more details.
;;;
;;;	You should have received a copy of the GNU General Public
;;;	License along with this program; if not, write to the Free
;;;	Software Foundation, Inc., 59 Temple Place - Suite 330,
;;;	Boston, MA 02111-1307, USA.

(declare (usual-integrations))

(define (make-new-world-from-old oldworld a-locations a-targets)
  (let* ((actuators (world-actuators oldworld))
	 (newworld (make-world (world-name oldworld)
			       (world-next-state-var-counter oldworld)
			       (world-initial-state oldworld)
			       '() ;;actuators
			       (world-probes oldworld)
			       (world-topology oldworld)
			       (world-latest-state oldworld)
			       (world-state-history oldworld))))
    (for-each (lambda (actuator location target)
		(let ((new-actuator (make-custom-actuator (actuator-name actuator)
							  location
							  target
							  (actuator-invert-polarity-p actuator)
							  (actuator-magnet actuator))))
		  (add-actuator-to-world! newworld new-actuator)))
	      (reverse actuators)
	      (reverse a-locations)
	      (reverse a-targets))
    newworld))

(define (optimize-targeting-1-actuator world #!optional location min-target-x-offset max-target-x-offset)
  (if (default-object? location)
      (set! location (actuator-location (car (world-actuators world)))))
  (if (default-object? min-target-x-offset)
      (set! min-target-x-offset *optimize-default-min-target-x-offset*))
  (if (default-object? max-target-x-offset)
      (set! max-target-x-offset *optimize-default-max-target-x-offset*))
  (minimize (lambda (x-target-offset)
	      (let* ((newworld (make-new-world-from-old world
						       (list location)
						       (list (3point-+ (actuator-target-for-orientation (car (world-actuators world)))
								       (make-3point x-target-offset 0 0)))))
		     (characterization (characterize-world newworld
							   *char-max-sideaccel*
							   *char-min-prop-accel*
							   *char-max-prop-accel*
							   #f 
							   'quiet-mode))
		     ;;account for non-existent region size due to excessive forces
		     ;;use of x-target-offset as minimization value ensures minimization of angle when region is non-existent
		     ;;basically makes plot look better - and any value is Ok since it is an unattainable data point
		     (effective-size (compute-effective-operating-size-for-minimization characterization
											(abs x-target-offset) ;for no-region, use of positive x-target-offset minimizes angle
											))
		     )
		effective-size))
	    *optimize-default-min-target-x-offset*
	    *optimize-default-max-target-x-offset*))

(define (compute-effective-operating-size-for-minimization characterization value-to-use-for-nonexisting-region)
  (if (and (number? (world-characterization-operating-min characterization))
	   (number? (world-characterization-operating-max characterization))
	   (number? (world-characterization-operating-size characterization)))
      (- (world-characterization-operating-size
	  characterization)) ;;use negative of region-size as cost value for minimization
      value-to-use-for-nonexisting-region))

;;optimize actuator offset location, using the best orientation at each candidate location

(define (optimize-offset-and-targeting-1-actuator world)
  ;;Scan the candidate locations and choose the best based on region-size
  (let* ((optimized-location-result
	  (minimize (lambda (x-location-offset) ;;run outer optimization loop
		      (let* ((location (3point-+ (actuator-location (car (world-actuators world)))
						 (make-3point x-location-offset
							      0
							      0)))
			     ;;for each candidate location, select the best orientation
			     ;;and use the region-size associated with that orientation
			     (optimized-target-result (optimize-targeting-1-actuator world location))
			     (neg-region-size (cadr optimized-target-result)))
			neg-region-size))
		    *optimize-min-location-offset*
		    *optimize-max-location-offset*
		    ))
	 (optimized-location-offset (car optimized-location-result))
	 (optimized-location (3point-+ (actuator-location (car (world-actuators world)))
				       (make-3point optimized-location-offset
						    0
						    0)))
	 (optimized-target-result-at-optimized-location (optimize-targeting-1-actuator world optimized-location))
	 (optimized-target-offset (car optimized-target-result-at-optimized-location))
	 )
    (list (cons 'location-offset optimized-location-result)
	  (cons 'target-offset optimized-target-result-at-optimized-location))
    ))

(define (plot-actuator-location-offset-vs-optimal-target-offset world)
  (let* ((location-x-values (enumerate-interval *optimize-min-location-offset*
						*optimize-max-location-offset*
						(/ (- *optimize-max-location-offset*
						      *optimize-min-location-offset*)
						   100)))
	 (target-x-offset-values (map (lambda (x-location-offset)
					(let* ((location (3point-+ (actuator-location (car (world-actuators world)))
								   (make-3point x-location-offset 0 0)))
					       (optimized-target-result (optimize-targeting-1-actuator world location))
					       (target-offset (car optimized-target-result))
					       (neg-region-size (cadr optimized-target-result)))
					  (if (or (not (number? neg-region-size))
						  (positive? neg-region-size))
					      *noplot*
					      target-offset)))
				      location-x-values))
	 (vals (map list
		    location-x-values 
		    target-x-offset-values)))
    (smart-gnuplot first second vals
		   (compose-sidewall-text (world-name world))
		   "Location Offset" "Target Offset" 'lines)))

(define (compose-sidewall-text msg)
  (format #f "~a ~a"
	  (if *sidewall-constraint-active*
	      "With Sidewall Constraint"
	      "WITHOUT-SIDEWALL-CONSTRAINT"
	      )
	  msg))

(define (plot-actuator-location-offset-vs-RELATIVE-target-offset world)
  (let* ((location-x-values (enumerate-interval *optimize-min-location-offset*
						*optimize-max-location-offset*
						(/ (- *optimize-max-location-offset*
						      *optimize-min-location-offset*)
						   100)))
	 (target-x-offset-values (map (lambda (x-location-offset)
					(let* ((location (3point-+ (actuator-location (car (world-actuators world)))
								   (make-3point x-location-offset 0 0)))
					       (optimized-target-result (optimize-targeting-1-actuator world location))
					       (target-offset (car optimized-target-result))
					       (neg-region-size (cadr optimized-target-result)))
					  (if (or (not (number? neg-region-size))
						  (positive? neg-region-size))
					      *noplot*
					      (- target-offset x-location-offset) ;;RELATIVE
					      )))
				      location-x-values))
	 (vals (map list
		    location-x-values 
		    target-x-offset-values)))
    (smart-gnuplot first second vals (compose-sidewall-text (world-name world)) "Location Offset" "RELATIVE Target Offset" 'lines)))

(define (plot-actuator-location-offset-vs-region-size world)
  (let* ((location-x-values (enumerate-interval *optimize-min-location-offset*
						*optimize-max-location-offset*
						(/ (- *optimize-max-location-offset*
						      *optimize-min-location-offset*)
						   100)))
	 (size-values (map (lambda (x-location-offset)
			     (let* ((location (3point-+ (actuator-location (car (world-actuators world)))
							(make-3point x-location-offset 0 0)))
				    (optimized-target-result (optimize-targeting-1-actuator world location))
				    (neg-region-size (cadr optimized-target-result)))
			       (if (or (not (number? neg-region-size))
				       (positive? neg-region-size))
				   *noplot*
				   (- neg-region-size))))
			   location-x-values))
	 (vals (map list
		    location-x-values 
		    size-values))
	 )
    (smart-gnuplot first second vals (compose-sidewall-text (world-name world)) "Actuator Location" "Region Size" 'lines)))

(define (plot-actuator-location-offset-vs-characterization-variable world characterization-accessor #!optional gnuplot-interface-filename)
  (plot-actuator-location-offset-vs-characterization-variable-selective-rotation
   world characterization-accessor #t gnuplot-interface-filename))

(define (plot-actuator-location-offset-vs-characterization-variable-no-rotation world characterization-accessor #!optional gnuplot-interface-filename)
  (plot-actuator-location-offset-vs-characterization-variable-selective-rotation
   world characterization-accessor #f gnuplot-interface-filename))

(define (plot-actuator-location-offset-vs-characterization-variable-selective-rotation world characterization-accessor rotation-p #!optional gnuplot-interface-filename)
  (let* ((location-x-values (enumerate-interval *optimize-min-location-offset*
						*optimize-max-location-offset*
						(/ (- *optimize-max-location-offset*
						      *optimize-min-location-offset*)
						   100)))
	 (result-values
	  (map (lambda (x-location-offset)
		 (let* ((location (3point-+ (actuator-location (car (world-actuators world)))
					    (make-3point x-location-offset 0 0)))
			(optimized-target-result (optimize-targeting-1-actuator world location))
			(target-offset (car optimized-target-result))
			(target (3point-+ (actuator-target-for-orientation (car (world-actuators world)))
					  (if rotation-p
					      (make-3point target-offset 0 0)
					      (make-3point x-location-offset 0 0))))
			(newworld (make-new-world-from-old world
							   (list location)
							   (list target)))
			(characterization
			 (characterize-world newworld
					     *char-max-sideaccel*
					     *char-min-prop-accel*
					     *char-max-prop-accel*
					     #f 
					     'quiet-mode)))
		   (characterization-accessor characterization)))
	       location-x-values))
	 (vals (map list
		    location-x-values 
		    result-values))
	 )
    (smart-gnuplot first second vals (compose-sidewall-text (world-name world))
		   (format #f "Location Offset (Rotation: ~a)" rotation-p)
		   (procedure-name characterization-accessor) 'lines gnuplot-interface-filename)))

;;to get a feeling for what rotation does
(define (plot-actuator-target-offset-vs-characterization-variable world characterization-accessor #!optional  x-location-offset relative-p gnuplot-interface-filename)
  (if (default-object? x-location-offset)
      (set! x-location-offset 0.))
  (if (default-object? relative-p)
      (set! relative-p #f))
  (let* ((location (3point-+ (actuator-location (car (world-actuators world)))
			     (make-3point x-location-offset 0 0)))
	 (target-offset-values (enumerate-interval *optimize-default-min-target-x-offset*
						   *optimize-default-max-target-x-offset*
						   (/ (- *optimize-default-max-target-x-offset*
							 *optimize-default-min-target-x-offset*)
						      200)))
	 (result-values (map (lambda (target-offset)
			       (let* ((target (3point-+ (actuator-target-for-orientation (car (world-actuators world)))
							(make-3point target-offset 0 0)))
				      (newworld (make-new-world-from-old world
									 (list location)
									 (list target)))
				      (characterization
				       (characterize-world newworld
							   *char-max-sideaccel*
							   *char-min-prop-accel*
							   *char-max-prop-accel*
							   #f 
							   'quiet-mode)))
				 (characterization-accessor characterization)))
			     target-offset-values))
	 (processed-target-offset-values
	  (map (lambda (offset) (if relative-p
				    (- offset x-location-offset)
				    offset))
	       target-offset-values))
	 (vals (map list
		    processed-target-offset-values
		    result-values))
	 )
    (smart-gnuplot first second vals (compose-sidewall-text (world-name world))
		   (format #f "~a Target Offset (location offset: ~a)"
			   (if relative-p "RELATIVE" "")
			   x-location-offset)
		   (procedure-name characterization-accessor) 'lines gnuplot-interface-filename)))


#|
;;operating-size
(plot-actuator-location-offset-vs-characterization-variable testtube-world-10
							    (named-lambda (worldchar-op-size c)
							      (let ((size (world-characterization-operating-size c)))
								(if (or (not (number? size))
									(negative? size))
								    *noplot*
								    size))))

;;operating min
(plot-actuator-location-offset-vs-characterization-variable testtube-world-10
							    (named-lambda (worldchar-op-min c)
							      (let ((size (world-characterization-operating-size c)))
								(if (or (not (number? size))
									(negative? size))
								    *noplot*
								    (world-characterization-operating-min c)))))

;;operating max
(plot-actuator-location-offset-vs-characterization-variable testtube-world-10
							    (named-lambda (worldchar-op-max c)
							      (let ((size (world-characterization-operating-size c)))
								(if (or (not (number? size))
									(negative? size))
								    *noplot*
								    (world-characterization-operating-max c)))))

(plot-actuator-location-offset-vs-optimal-target-offset testtube-world-10)
(plot-actuator-location-offset-vs-RELATIVE-target-offset testtube-world-10)
|#
