;;; The contents of this directory structure is subject to the following
;;; copyright notice:
;;;
;;;	Copyright (c) 2006, 2007 Andrew A. Berlin
;;;
;;;	This program is free software; you can redistribute it and/or
;;;	modify it under the terms of the GNU General Public License as
;;;	published by the Free Software Foundation; either version 2 of
;;;	the License, or (at your option) any later version.
;;;
;;;	This program is distributed in the hope that it will be
;;;	useful, but WITHOUT ANY WARRANTY; without even the implied
;;;	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
;;;	PURPOSE.  See the GNU General Public License for more details.
;;;
;;;	You should have received a copy of the GNU General Public
;;;	License along with this program; if not, write to the Free
;;;	Software Foundation, Inc., 59 Temple Place - Suite 330,
;;;	Boston, MA 02111-1307, USA.

(declare (usual-integrations))

;; Plots a heatmap of forces/torques within a particular radius 
;; of a point alpha within a FT.  
;; Cross section is taken perpendicular to tube.

;;direction is Fx, Fy, Fz
;;perpendicular direction is described by two perpendicular vectors:
;;one is (-y, x) and the other is (-z, x)

(define *debug-offsets* '())
(define *debug-h1*)
(define *debug-h2*)

(define (find-h1 pos dir)
    (let* ((a (3point-x dir))
	   (b (3point-y dir))
	   (c (3point-z dir))
	   (d (- (3point-dot-product dir pos))))
      (cond ((and (= d 0) ;;if d is zero and pos is not, pos can serve as h1^
		  (not (3point-zero? pos)))
	     (scale-3point-to-unit-vector pos))
	    ((and (= d 0)
		  (not (= a 0)))
	     (make-unit-3point (/ (- b) a)
			       1
			       0))
	    ((and (= d 0)
		  (not (= b 0)))
	     (make-unit-3point 0
			       (/ (- c) b)
			       1))
	    ((and (= d 0)
		  (not (= c 0)))
	     (make-unit-3point 1
			       0
			       (/ (- a) c)))

	    ((= d 0)
	     (error "plot-heatmap: a, b, c, and d are all 0"))

	    
	    ;;now handle case where d is non-zero
	    ;;that meaans there is a component of pos in direction dir
	    ;;if tube direction includes the Z-axis, use the
	    ;;Z axis intercept (x=y=0) of the plane 
	    ;;unless pos is the same point
	    ((and (not (= c 0)) 
		  (not (and (= (3point-x pos) 0)
			    (= (3point-y pos) 0)
			    (= (3point-z pos) (/ (- d) c)))))
	     (compute-3pt-orientation pos
				      (make-3point 0
						   0
						   (/ (- d) c))))
	    ((not (= c 0)) ;;c is not 0 so choose x=y=1 as intercept:
	     (compute-3pt-orientation pos
				      (make-3point 1
						   1
						   (/ (- (+ a b d))
						      c))))

	    ;;if tube direction does not include the Y-axis, use the
	    ;;Y axis intercept (x=y=0) as the point to define the plane
	    ;;unless pos is the same point - then use x=z=1
	    ((and (not (= b 0))
		  (not (and (= (3point-x pos) 0)
			    (= (3point-y pos) (/ (- d) b))
			    (= (3point-z pos) 0))))
	     (compute-3pt-orientation pos
				      (make-3point 0
						   (/ (- d) b)
						   0)))
	    ((not (= b 0))
	     (compute-3pt-orientation pos
				      (make-3point 1
						   (/ (- (+ a c d)) b)
						   1)))
	    ((and (not (= a 0))
		  (not (and (= (3point-x pos) (/ (- d) a))
			    (= (3point-y pos) 0)
			    (= (3point-z pos) 0))))
	     (compute-3pt-orientation pos
				      (make-3point (/ (- d) a)
						   0
						   0)))
	    ((not (= a 0))
	     (compute-3pt-orientation pos
				      (make-3point (/ (- (+ b c d)) a)
						   1
						   1)))
	    (else
	     (error "Invalid tube direction detected" (list dir pos)))
	    )))

;;f is a function that takes a 3point and returns record to plot
(define (plot-heatmap world alpha radius f #!optional increment outer-increment)
  (if (default-object? increment)
      (set! increment (/ radius *default-heatmap-granularity*)))
  (if (default-object? outer-increment)
      (set! outer-increment
	    (/ radius *default-heatmap-granularity-outer-increment*)))
  (let* ((topology (world-topology world))
	 (pos-f (topology-pos-f topology))
	 (dir-f (topology-dir-f topology))
	 (pos (pos-f alpha topology))
	 (dir (dir-f alpha topology))
	 (h1^ (find-h1 pos dir))
	 (h2^ (scale-3point-to-unit-vector (3point-cross-product h1^ dir))))
    (define (process-radius r)
      (let ((upper-half-plane 
	     (map (lambda (h1)
		    (let* ((h2 (sqrt (- (square r)
					(square h1))))
			   (point-to-plot
			    (3point-+ (scale-3point h1 h1^)
				      (scale-3point h2 h2^))))
		      point-to-plot))
		  (append
		   (enumerate-weighted-interval (- r) 0
						(/ increment 2)
						*circle-plot-weighting-region*
						*circle-plot-focus-amount*)
		   (enumerate-weighted-interval 0 r
						(* (/ increment 2)
						   *circle-plot-focus-amount*)
						(- 1 *circle-plot-weighting-region*)
						(/ *circle-plot-focus-amount*))
		   )))
	    (lower-half-plane
	     (map (lambda (h1)
		    (let* ((h2 (sqrt (- (square r)
					(square h1))))
			   (point-to-plot
			    (3point-+ (scale-3point h1 h1^)
				      (scale-3point (- h2) h2^))))
		      point-to-plot))
		  (append
		   (enumerate-weighted-interval (- r) 0
						(/ increment 2)
						*circle-plot-weighting-region*
						*circle-plot-focus-amount*)

		   (enumerate-weighted-interval 0 r
						(* (/ increment 2)
						   *circle-plot-focus-amount*)
						(- 1 *circle-plot-weighting-region*)
						(/ *circle-plot-focus-amount*))

		   ))
	     ))
	(append lower-half-plane
		upper-half-plane)))
;;    (set! *debug-h1* h1^)
;;    (set! *debug-h2* h2^)
    (let* ((pos-offset-points-dataset
	    (apply append
		   (map (lambda (r)
			  (process-radius r))
			(enumerate-interval 0 radius outer-increment)
			;;for debugging, to get just circle:
			;;(enumerate-interval radius radius outer-increment) 
			)))
	   (f-dataset (map f pos-offset-points-dataset)))
      (plot-fallopian-tube-profile-dataset-csv f-dataset)
;;      (set! *debug-offsets* (list pos-offset-points-dataset f-dataset))
      )))
      
;;gets more dense towards end in order to account for use of 
;;sqrt on the results, which compresses the datapoints
(define (enumerate-weighted-interval start end increment tran-mult focus-mult)
  (let ((transition (+ start (* tran-mult (- end start))))
	(weighted-increment (* increment focus-mult)))
    (append (enumerate-interval start transition weighted-increment)
	    (enumerate-interval transition
				end
				increment))
    ))



;;For debugging:

;;checks that pt is member of orthogonality plane at alpha
;;should = 0  if orthogonal
(define (check-for-orthogonality pt alpha world)
  (let* ((topo (world-topology world))
	 (pos-f (topology-pos-f topo))
	 (dir-f (topology-dir-f topo))
	 (pos (pos-f alpha topo))
	 (dir (dir-f alpha topo)))
    (3point-dot-product dir
			(3point-- pt pos))))
	      
(define (make-os world)
  (let ((f-dataset (second *debug-offsets*)))
    (map (lambda (thing)
	   (check-for-orthogonality (fallopian-tube-force-profile-3pt-location thing)
				    (fallopian-tube-force-profile-alpha thing)
				    example-hlh-world))
	 f-dataset))			
  )
  