;;; The contents of this directory structure is subject to the following
;;; copyright notice:
;;;
;;;	Copyright (c) 2006, 2007 Andrew A. Berlin
;;;
;;;	This program is free software; you can redistribute it and/or
;;;	modify it under the terms of the GNU General Public License as
;;;	published by the Free Software Foundation; either version 2 of
;;;	the License, or (at your option) any later version.
;;;
;;;	This program is distributed in the hope that it will be
;;;	useful, but WITHOUT ANY WARRANTY; without even the implied
;;;	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
;;;	PURPOSE.  See the GNU General Public License for more details.
;;;
;;;	You should have received a copy of the GNU General Public
;;;	License along with this program; if not, write to the Free
;;;	Software Foundation, Inc., 59 Temple Place - Suite 330,
;;;	Boston, MA 02111-1307, USA.

;;;Need to add viscous drag proportional to velocity and proportional to curvature of tube

(declare (usual-integrations))

(load-option 'format)

;;configure integrator in scmutils
(set! *compiling-sysder? #f)

;;utilities

(define (square x) (* x x))

(define (gauss->tesla gauss)
  (/ gauss 1e4))

(define (in->m inches)
  (/ (* inches 2.54) 100.0))
(define (inches->m inches)
  (in->m inches))

(define (m->inches m)
  (* 100.0 (/ m 2.54)))
(define (m->in m)
  (m->inches m))
(define (mm->m mm)
  (/ mm 1000))

(define (cm->m cm)
  (/ cm 100))

(define (make-unit-3point x y z)
  (let ((len (sqrt (+ (square x)
		      (square y)
		      (square z)))))
    (make-3point (/ x len)
		 (/ y len)
		 (/ z len))))

(define (scale-3point-to-unit-vector 3pt)
  (make-unit-3point (3point-x 3pt)
		    (3point-y 3pt)
		    (3point-z 3pt)))

(define (cylinder-volume len diameter)
  (let ((r (/ diameter 2)))
    (* pi (square r) len)))

;;constants
(define u0 (* 4 pi 1e-7)) ;;Newtons/Ampere^2 or Tesla meters/ampere
(define g 9.8) ;;meters/second
(define pi 3.1415927)
(define pi/2 (/ pi 2))

;;structures
(define-structure 3point x y z)

(define (make-3point-inches x y z)
  (make-3point (in->m x)
	    (in->m y)
	    (in->m z)))

(define (3point->list p)
  (list (3point-x p)
	(3point-y p)
	(3point-z p)))

(define-structure dipole
  name
  halflen ;;meters
  strength ;; Tesla/ampere
  geometry
  )

(define-structure monopole strength location)

(define-structure magnet name geometry dipole)

(define (monopole-monopole-force-mag m1 m2)
  (/ (* u0 (monopole-strength m1) (monopole-strength m2))
     (* 4 pi (square (monopole-separation m1 m2)))))

(define (monopole-separation m1 m2)
  (let ((l1 (monopole-location m1))
	(l2 (monopole-location m2)))
    (sqrt (+ (square (- (3point-x l1) (3point-x l2)))
	     (square (- (3point-y l1) (3point-y l2)))
	     (square (- (3point-z l1) (3point-z l2)))
	     )))) 

(define (monopole-force-direction m1 m2)
  (compute-3pt-orientation (monopole-location m1)
			   (monopole-location m2)))

(define-structure cylindrical-geometry
  name
  len
  diameter
  volume
  mass
  density
  Br
  strength)
    
(define (geometry-mass g)
  (cond ((cylindrical-geometry? g)
	 (cylindrical-geometry-mass g))
	(else
	 (error "Unknown geometry type"))))

(define (pt-above amount pt)
  (make-3point (3point-x pt)
	       (3point-y pt)
	       (+ (3point-z pt) amount)
	       ))

(define (pt-below amount pt)
  (pt-above (- amount) pt))

(define (pt-to-right amount pt)
  (make-3point (+ (3point-x pt) amount)
	       (3point-y pt)
	       (3point-z pt)))

(define (pt-to-left amount pt)
  (pt-to-right (- amount) pt))

(define (pt-towards-head amount pt)
  (make-3point  (3point-x pt)
		(+ (3point-y pt) amount)
		(3point-z pt)))
(define (pt-towards-feet amount pt)
  (pt-towards-head (- amount) pt))

(define origin (make-3point 0 0 0))

;; pt1 - pt2
(define (3point-- pt1 pt2)
  (make-3point (- (3point-x pt1) (3point-x pt2))
	       (- (3point-y pt1) (3point-y pt2))
	       (- (3point-z pt1) (3point-z pt2))))

(define (3point-+ pt1 pt2 . rest)
  (let ((first-sum (make-3point (+ (3point-x pt1) (3point-x pt2))
				(+ (3point-y pt1) (3point-y pt2))
				(+ (3point-z pt1) (3point-z pt2)))))
    (if (null? rest)
	first-sum
	(apply 3point-+
	       (cons first-sum rest)))))

(define (scale-3point amt pt)
  (make-3point (* (3point-x pt) amt)
	       (* (3point-y pt) amt)
	       (* (3point-z pt) amt)))

(define (3point-dot-product pt1 pt2)
  (+ (* (3point-x pt1) (3point-x pt2))
     (* (3point-y pt1) (3point-y pt2))
     (* (3point-z pt1) (3point-z pt2))))

(define (3point-cross-product pt1 pt2)
  (let ((a1 (3point-x pt1))
	(a2 (3point-y pt1))
	(a3 (3point-z pt1))
	(b1 (3point-x pt2))
	(b2 (3point-y pt2))
	(b3 (3point-z pt2)))
    (make-3point (+ (* (- a3) b2) (* a2 b3))
		 (+ (* a3 b1) (* (- a1) b3))
		 (+ (* (- a2) b1) (* a1 b2)))))

(define (3point-distance pt1 pt2)
  (3point-magnitude (3point-- pt2 pt1)))

(define (3point-magnitude pt)
  (sqrt (3point-dot-product pt pt)))

(define (3point-zero? pt)
  (and (= (3point-x pt) 0)
       (= (3point-y pt) 0)
       (= (3point-z pt) 0)))

;;returns unit vector from from pt to to pt
(define (compute-3pt-orientation from to)
  (let ((pt (3point-- to from)))
    (let ((result (make-unit-3point (3point-x pt)
				    (3point-y pt)
				    (3point-z pt))))
      ;;      (display 'compute-3pt-orientation)
      ;;      (newline)
      ;;      (pp  result)
      result)))

(define standard-magnet-length (mm->m 37))
(define standard-magnet-diameter (mm->m 37))
(define standard-magnet-mass 0.3) ;Kg
(define standard-magnet-Br 1.2650) ;Tesla

;;target-for-orientation is an absolute coordinate
;;pass in magnet if want to override standard values
(define (make-custom-actuator name position-3pt target-for-orientation #!optional invert-polarity-p magnet)
  (if (default-object? invert-polarity-p)
      (set! invert-polarity-p false))
  (let* ((orientation (if invert-polarity-p
			  (compute-3pt-orientation target-for-orientation position-3pt) ;;flip from and to
			  (compute-3pt-orientation position-3pt target-for-orientation)))
	 (magnet (if (default-object? magnet)
		     (make-cylindrical-magnet name
					      standard-magnet-mass
					      standard-magnet-length
					      standard-magnet-diameter
					      standard-magnet-Br)
		     magnet))
	 (dipole (magnet-dipole magnet))
	 (strength (dipole-strength dipole))
	 (halflen (dipole-halflen dipole))
	 (monopole1-location (3point-+ position-3pt
				       (scale-3point halflen orientation)))
	 (monopole2-location (3point-+ position-3pt
				       (scale-3point (- halflen) orientation)))
				       
	 (monopole1 (make-monopole strength monopole1-location))
	 (monopole2 (make-monopole (- strength) monopole2-location)))
    (make-actuator name position-3pt target-for-orientation invert-polarity-p magnet monopole1 monopole2)))

(define (make-cylindrical-magnet name mass len diameter Br)
  (let* ((volume (cylinder-volume len diameter))
	 (density (/ mass volume))
	 (magmoment (* Br volume (/ u0)))
	 (strength (/ magmoment len))
	 (geometry (make-cylindrical-geometry name
					      len
					      diameter
					      volume
					      mass
					      density
					      Br
					      strength))
	 (halflen (/ len 2))
	 (dipole (make-dipole name halflen strength geometry))
	 (magnet (make-magnet name geometry dipole)))
    magnet))

;;;pos-f takes as argument position alpha along the tube and returns the 3point coordinate of the tube at that point
;;;dir-f takes as argument position alpha along the tube and returns the unit vector direction of the tube
;;;L is length of tube
(define-structure topology name startpt deltapt pos-f dir-f L misc)

(define (make-fallopian-tube name startpt endpt pos-f dir-f L . misc)
  (let* ((deltax (- (3point-x endpt) (3point-x startpt)))
	 (deltay (- (3point-y endpt) (3point-y startpt)))
	 (deltaz (- (3point-z endpt) (3point-z startpt)))
	 (deltapt (make-3point deltax deltay deltaz)))
    (make-topology name
		   startpt
		   deltapt
		   pos-f
		   dir-f
		   L
		   misc)))


#|
;;;World is basis for simulation
;;;World holds all static variable descriptions of the system
;;;State vector holds all dynamic variables
;;;World assigns indeces in the state vector for each dynamic variable
|#

(define-structure world
  name
  next-state-var-counter
  initial-state
  actuators
  probes
  topology
  latest-state
  state-history)
  
;;state-var-counter 0 is reserved for time

;;returns state vector index of next available state variable
(define (world-allocate-state-variable! world)
  (let ((var-reference (world-next-state-var-counter world)))
    (set-world-next-state-var-counter! world
				       (1+ var-reference))
    ;;grow initial-state vector to make room for the new variable
    (set-world-initial-state! world
			      (vector-append (world-initial-state world)
					     #(uninitialized-state-variable)))
    var-reference))

;;probes are free magnets that move relative to fallopian tube
;;position-reference and velocity-reference are indeces into state vector
(define-structure probe
  name
  position-reference
  velocity-reference
  magnet)

;;initial-location and initial-velocity are scalars reflecting position along tube.
;;returns probe
(define (make-probe-and-add-to-world! world name initial-location initial-velocity magnet)
  (let* ((pos-reference (world-allocate-state-variable! world))
	 (vel-reference (world-allocate-state-variable! world))
	 (probe (make-probe name pos-reference vel-reference magnet))
	 (initial-state-vector (world-initial-state world)))
    ;;add probe to world
    (set-world-probes! world
		       (cons probe
			     (world-probes world)))
    ;;add initial-state information to world
    (vector-set! initial-state-vector pos-reference initial-location)
    (vector-set! initial-state-vector vel-reference initial-velocity)
    probe
    ))
			 
;;for now actuators do not have state associated with them: fixed in place
(define (add-actuator-to-world! world actuator)
  (set-world-actuators! world
			(cons actuator
			      (world-actuators world)))
  actuator)


(define (make-sysder world)
  ;;sysder is a function that returns a function that takes state as argument and does derivative
  ;;done this way per refman.txt
  (define (sysder)
    (define (derivative-provider state)
      (let  ((new-state (make-vector (vector-length state) 'uninitialized-from-make-sysder)))
;;	(format #t "entering derivative ~a" state)
;;	(newline)
	(vector-set! new-state 0 1.0) ;;derivative of time is 1.0 by definition
	(for-each (lambda (probe)
		    (update-probe-deriv! world probe state new-state))
		  (world-probes world))
;;	(format #t "exiting derivative ~a" new-state)
;;	(newline)
	new-state))
    derivative-provider) ;;sysder returns derivative-provider
  sysder) ;;make-sysder returns sysder

(define (probe-mass probe)
  (geometry-mass (magnet-geometry (probe-magnet probe))))

(define (update-probe-deriv! world probe state deriv-state)
  (let* ((probe-pos-ref (probe-position-reference probe))
	 (probe-vel-ref (probe-velocity-reference probe)))
    ;;derivative of position is velocity
    (vector-set! deriv-state		
		 probe-pos-ref		;derivative of position is velocity
		 (vector-ref state probe-vel-ref)) ;which is available in the current state
    ;Derivative of velocity is acceleration
    ;Acceleration = F/M
    (let* ((tf (compute-tf-collection world probe state (make-3point 0 0 0)))
	   (Fprop-mag (torque-and-forces-collection-Fpropulsive-mag tf))
	   (mass (probe-mass probe))
	   (acceleration (/ Fprop-mag mass)))
      (vector-set! deriv-state
		   probe-vel-ref
		   acceleration)
      acceleration)))

(define-structure actuator name location target-for-orientation invert-polarity-p magnet monopole1 monopole2)

;;probe-orientation is a unit vector
(define (force-component-along-probe-orientation force-vector probe-orientation)
  (3point-dot-product force-vector probe-orientation))

(define (compute-probe-3pt-position world alpha) ;;alpha is position along length of fallopian tube
  (let* ((topology (world-topology world))
	 (pos-f (topology-pos-f topology)))
    (pos-f alpha topology)))

;;returns a unit vector
(define (compute-probe-orientation world alpha) ;;alpha is position along length of fallopian tube
  (let* ((topology (world-topology world))
	 (dir-f (topology-dir-f topology)))
    (dir-f alpha topology)))

;;returns list of monopole1 and monopole2
(define (make-probe-monopoles world probe state pos-offset)
  (let* ((probe-position-alpha (vector-ref state (probe-position-reference probe))) ;;probe position alpha in fallopian tube
	 (probe-position-3pt
	  (compute-probe-3pt-position world probe-position-alpha)) ;;probe position in 3 space
	 (probe-orientation (compute-probe-orientation world probe-position-alpha)) ;;probe orientation unit vector, pointed towards fallopian tube forward progress
	 )
    
    (make-probe-monopoles-from-pos-orientation probe
					       (3point-+ probe-position-3pt
							 pos-offset)
					       probe-orientation)))

(define (make-probe-monopoles-from-pos-orientation probe pos orientation)
    (let* ((offset (dipole-halflen (magnet-dipole (probe-magnet probe))))
	   (strength (dipole-strength (magnet-dipole (probe-magnet probe))))
	   
	   (probe-m1-location (3point-+ pos
					(scale-3point offset orientation)))
	   (probe-m2-location (3point-+ pos
					(scale-3point (- offset) orientation)))
	   (p-m1 (make-monopole strength
				probe-m1-location)) ;;probe monopole 1
	   (p-m2 (make-monopole (- strength)
				probe-m2-location)) ;;probe monopole 2
	   )
      (list p-m1 p-m2)))

;;first computes force acting on monopole1 in direction of monopole2
;;then computes component of that force that is aligned with orientation
;;orientation is a unit vector
(define (oriented-monopole-monopole-force-mag orientation monopole1 monopole2)
  (3point-dot-product (total-monopole-monopole-force monopole1 monopole2)
		      orientation))

(define (total-monopole-monopole-force monopole1 monopole2)
  (let* ((force-magnitude (monopole-monopole-force-mag monopole1 monopole2))
	 (force-direction (monopole-force-direction monopole1 monopole2)) ;;unit vector
	 (force-vector (scale-3point force-magnitude force-direction)))
    force-vector))

(define (run-simulator world end-time noisy?)
  (newline)

  ;;prepare to record the results
  (set-world-latest-state! world (world-initial-state world))
  (set-world-state-history! world '())
  
  ;;start the integration
  (let ((sysder (make-sysder world)))
    ((evolve sysder)
     (world-initial-state world)
     (named-lambda (sim-integration-monitor state)
       (integration-monitor world state noisy?))
     dt
     end-time
     eps))

  ;;return world as result.
  ;;world will contain integration state results in world-state-history and in 
  ;;world-latest-state and via the integration-monitor
  world)

(define (integration-monitor world state noisy?)
  (set-world-latest-state! world state)
  (set-world-state-history! world (cons state (world-state-history world)))
  (if noisy?
      (pp state))
  state)

(define (make-new-world topology #!optional name)
  (if (default-object? name)
      (set! name 'world))
  (let ((initial-state #(0)))
    (make-world name
		1 ;;state-var-counter
		initial-state ;;initial-state.  time starts at 0
		'() ;;actuators
		'() ;;probes
		topology
		'uninitialized-latest-state 
		'uninitialized-state-history
		))) ;;state-history

