;;; The contents of this directory structure is subject to the following
;;; copyright notice:
;;;
;;;	Copyright (c) 2006, 2007 Andrew A. Berlin
;;;
;;;	This program is free software; you can redistribute it and/or
;;;	modify it under the terms of the GNU General Public License as
;;;	published by the Free Software Foundation; either version 2 of
;;;	the License, or (at your option) any later version.
;;;
;;;	This program is distributed in the hope that it will be
;;;	useful, but WITHOUT ANY WARRANTY; without even the implied
;;;	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
;;;	PURPOSE.  See the GNU General Public License for more details.
;;;
;;;	You should have received a copy of the GNU General Public
;;;	License along with this program; if not, write to the Free
;;;	Software Foundation, Inc., 59 Temple Place - Suite 330,
;;;	Boston, MA 02111-1307, USA.

(declare (usual-integrations))

;;for debugging

;;assume that b is ground truth
(define (percent-difference a b)
  (* 100 (/ (- a b)
	    b)))

(define (describe-world w)
  (let* ((a (world-actuators w))
	 (probes (world-probes w))
	 (t (world-topology w))
	 (h (world-state-history w)))
    (format #t "~%World named ~a has topology ~a with ~a actuators, ~a probes, and ~a state history records"
	    (world-name w)
	    (topology-name t)
	    (length a)
	    (length probes)
	    (length h))
    (describe-topology t)
    (for-each (lambda (probe) (describe-probe w probe)) probes)
    (for-each describe-actuator a)
    (describe-state-history h)
    ))

(define (describe-actuator a)
  (let* ((m1 (actuator-monopole1 a))
	 (m2 (actuator-monopole2 a))
	 (m1-loc (monopole-location m1))
	 (m2-loc (monopole-location m2)))
    (format #t "~%Actuator ~a has monopoles at (~a, ~a, ~a) of strength ~a,~% and at (~a, ~a, ~a) of strength ~a"
	    (actuator-name a)
	    
	    (3point-x m1-loc)
	    (3point-y m1-loc)
	    (3point-z m1-loc)
	    (monopole-strength m1)
	    
	    (3point-x m2-loc)
	    (3point-y m2-loc)
	    (3point-z m2-loc)
	    (monopole-strength m2)
	    )))

(define (describe-probe w p)
  (let* ((m (probe-magnet p))
	 (s (last (world-state-history w))) ;;initial state
	 (p-m1 (first (make-probe-monopoles w p s (make-3point 0 0 0))))
	 (p-m2 (second (make-probe-monopoles w p s (make-3point 0 0 0)))))
    (format #t "~%Probe ~a has dipole of strength ~a and geometry as follows:"
	    (probe-name p)
	    (dipole-strength (magnet-dipole m))
	    )
    (pp (magnet-geometry m))
    (newline)
    (format #t "~%Probe ~a has monopoles located at (~a, ~a, ~a) of strength ~a,~% and at (~a, ~a, ~a) of strength ~a"
	    (probe-name p)
	    (3point-x (monopole-location p-m1))
	    (3point-y (monopole-location p-m1))
	    (3point-z (monopole-location p-m1))
	    (monopole-strength p-m1)

	    (3point-x (monopole-location p-m2))
	    (3point-y (monopole-location p-m2))
	    (3point-z (monopole-location p-m2))
	    (monopole-strength p-m2)
	    )
    ))

(define (describe-topology t)
  (let* ((startpt (topology-startpt t))
	 (deltapt (topology-deltapt t))
	 (endpt (3point-+ startpt deltapt)))
    (format #t "~%Topology ~a has startpoint (~a, ~a, ~a) and endpoint (~a, ~a, ~a)"
	    (topology-name t)
	    (3point-x startpt )
	    (3point-y startpt)
	    (3point-z startpt)

	    (3point-x endpt)
	    (3point-y endpt)
	    (3point-z endpt)
	    )))

(define (describe-state-history h)
  (format #t "~%State history has ~a entries covering time ~a through time ~a and position ~a through ~a"
	  (length h)
	  (state-time (last h))
	  (state-time (first h))
	  (apply min (map state-pos h))
	  (apply max (map state-pos h))
	  ))

(define (state-time s) (vector-ref s 0))
(define (state-pos s) (vector-ref s 1))
(define (state-vel s) (vector-ref s 2))

;;Test functions:

;;assumes gravity acts in the 'Y' direction only
;;returns the location alpha in the fallopian tube at which the 'Y' force balances
;;gravity
(define (compute-gravity-force-balance world #!optional in-water-p)
  (if (default-object? in-water-p)
      (set! in-water-p false))
  (if (> (length (world-probes world)) 1)
      (error "compute gravity force balance not implemented for multiple probes"))
  (let* ((p (first (world-probes world)))
	 (L (topology-L (world-topology world))) ;;length of tube
	 (g-force (net-gravity-force-mag-on-probe p in-water-p))) ;;directed downwards, i.e. negative
    (seek-target (- g-force) ;;seek 1g total y actuation force
		 (lambda (alpha)
		   ;;compute total y-directed magnetic forces
		   (let* ((y-unit-vector (make-unit-3point 0 1 0))
			  (ft (compute-fallopian-tube-forces-at-alpha world
								      alpha
								      (make-3point 0 0 0) ;pos-offset
								      ))
			  (Fprop-mag (fallopian-tube-force-profile-propulsive-force-mag ft))
			  (orientation (compute-probe-orientation world alpha))
			  (Fprop (scale-3point Fprop-mag orientation)) ;;propulsive force vector
			  (Fprop-y-directed-mag
			   (3point-dot-product Fprop y-unit-vector))
			  
			  (Ftrans (fallopian-tube-force-profile-translation-force ft))
			  (Ftrans-y-directed-mag (3point-dot-product Ftrans y-unit-vector))
			  (total-y-actuation-force-mag (+ Fprop-y-directed-mag
							  Ftrans-y-directed-mag)))
		     total-y-actuation-force-mag))
		 0
		 L)
    ))

#|old before seek-target was used:

(minimize (lambda (alpha)
		;;compute total y-directed magnetic forces
		(let* ((y-unit-vector (make-unit-3point 0 1 0))
		       (ft (compute-fallopian-tube-forces-at-alpha world
								   alpha
								   (make-3point 0 0 0) ;pos-offset
								   ))
		       (Fprop-mag (fallopian-tube-force-profile-propulsive-force-mag ft))
		       (orientation (compute-probe-orientation world alpha))
		       (Fprop (scale-3point Fprop-mag orientation)) ;;propulsive force vector
		       (Fprop-y-directed-mag
			(3point-dot-product Fprop y-unit-vector))

		       (Ftrans (fallopian-tube-force-profile-translation-force ft))
		       (Ftrans-y-directed-mag (3point-dot-product Ftrans y-unit-vector))
		       (total-y-actuation-force-mag (+ Fprop-y-directed-mag
						       Ftrans-y-directed-mag))
		       (total-plus-gravity-mag (+ total-y-actuation-force-mag g-force)))
		  (abs total-plus-gravity-mag))) ;;optimize to minimize total force incl. gravity.
	      0
	      L)
p
|#

(define (net-gravity-force-mag-on-probe probe #!optional in-water-p)
  (if (default-object? in-water-p)
      (set! in-water-p false))
  (let* ((m (probe-magnet probe))
	 (cg (magnet-geometry m))
	 (volume (cylindrical-geometry-volume cg))
	 (mass (probe-mass probe))
	 (density-of-water 1000)	;1 kg / 1000cc's = 1 kg * 1e3 = 1000

	 (gravity-acting-on-magnet (* -9.8 mass)) ;;directed downwards

	 (mass-of-displaced-water (* volume density-of-water))
	 (gravity-that-would-have-acted-on-water (* 9.8 mass-of-displaced-water)) ;;directed upwards

	 (net-gravity-acting-on-magnet (+ gravity-acting-on-magnet
					  gravity-that-would-have-acted-on-water)))
    (if in-water-p
	net-gravity-acting-on-magnet
	gravity-acting-on-magnet)))

;;(set! dt 0.001)
;;(format #t "~%~a" 
;;	#(time position velocity))
;;(run-simulator world *run-until* #t)

;;(define win (frame 0 3 -1 0))

(define (andy-frame . args)
  (format #t "~%Creating Frame ~a" args)
  (apply frame args))

;;returns window
(define (smart-plot ref1 ref2 states #!optional existing-win)
  (let* ((ref1s (map ref1 states))
	 (ref2s (map ref2 states))
	 (min-ref1 (apply min ref1s))
	 (max-ref1 (apply max ref1s))
	 (min-ref2 (apply min ref2s))
	 (max-ref2 (apply max ref2s))
	 
	 (win (if (default-object? existing-win)
		  (andy-frame min-ref1 max-ref1 min-ref2 max-ref2) ;;make a new window
		  existing-win)))

    (for-each (lambda (r1 r2)
		(plot-point win r1 r2))
	      ref1s
	      ref2s)
    (if *connect-dots-while-plotting*
	(smart-plot-connect-dots ref1 ref2 states win))
    win))

(define (smart-plot-connect-dots ref1 ref2 states win)
  (let loop ((first-state (car states))
	     (rest-states (cdr states)))
    (if (null? rest-states)
	win
	(let* ((first-of-rest (car rest-states))
	       (startpt-x (ref1 first-state))
	       (startpt-y (ref2 first-state))
	       (endpt-x (ref1 first-of-rest))
	       (endpt-y (ref2 first-of-rest)))
	  (plot-line win startpt-x startpt-y endpt-x endpt-y)
	  (loop first-of-rest (cdr rest-states))))))

(define (plot-pos states #!optional existing-win)
  (smart-plot state-time state-pos states existing-win))

(define (plot-vel states #!optional existing-win)
  (smart-plot state-time state-vel states existing-win))

(define (plot-phase-portrait states #!optional existing-win)
  (smart-plot state-pos state-vel states existing-win))

(define (plot-actuators-x world win)
  (for-each (lambda (a)
	      (plot-actuator-x a win))
	    (world-actuators world))
  win)

(define (draw-vert-line x-intercept win)
  (let ((coord-fcn (graphics-coordinate-limits win)))
    (coord-fcn
     (lambda (xleft ybottom xright ytop)
       (plot-line win x-intercept ybottom x-intercept ytop)))))

(define (draw-horiz-line y-intercept win)
  (let ((coord-fcn (graphics-coordinate-limits win)))
    (coord-fcn
     (lambda (xleft ybottom xright ytop)
       (plot-line win xleft y-intercept xright y-intercept)))))

(define (plot-actuator-x a win)
  (let* ((m1 (actuator-monopole1 a))
	 (m2 (actuator-monopole2 a))
	 (x1 (3point-x (monopole-location m1)))
	 (x2 (3point-x (monopole-location m2)))
	 (y1 (3point-y (monopole-location m1)))
	 (y2 (3point-y (monopole-location m2)))
	 (x (average x1 x2))
	 (y (average y1 y2))
;;	 (circlesize (abs (/ (- x2 x1) 2))) ;;kludge
	 (circlesize (/ x 16)) ;;kludge
	 )
#|
    (format #t "Plotting circle for actuator ~a win ~a at (~a, ~a) radius ~a"
	    (actuator-name a)
	    win
	    x
	    y
	    circlesize)
|#
;;    (graphics-operation win 'fill-circle x y circlesize)
    (draw-horiz-line x1 win)
    (draw-horiz-line x2 win)
    ))

;(define s (world-state-history world))
;(define vel-win (plot-vel s))
;(define pos-win (plot-pos s vel-win))
;;(define vel-win (plot-vel s))
;(define phase-win (plot-phase-portrait s))
;;(plot-actuators-x world pos-win)

;(describe-world world)

(define (log-base-10 x)
  (/ (log x)
     (log 10)))

(define (alog-base-10 x)
  (/ (log (abs x))
     (log 10)))

#|
Log X = answer
10^LogX = 10^answer
X = 10^answer
ln X = ln (10^answer) = answer * ln 10
answer = ln X / ln 10
|#


(define (main-probe-length world)
  (let* ((probe (first (world-probes world)))
	 (magnet (probe-magnet probe))
	 (dipole (magnet-dipole magnet))
	 (dipole-l (* 2 (dipole-halflen dipole))))
    dipole-l))

;;for debugging/checking against published paper
(define (force-on-axis-formula moment separation)
  (* (/ u0 2pi)
     (square moment)
     (/ 3 (expt separation 4))))

;;(define time-pos-forces (make-time-pos-forces world))

(define-structure fallopian-tube-force-profile
  alpha
  propulsive-force-mag 
  3pt-location
  propulsive-accel
  max-sidewall-force 
  max-sidewall-accel
  Torque
  translation-force-mag 
  translation-accel
  translation-force 
  rotation-force-mag 
  rotation-accel
  time ;;time and vel used by plot-simulation-results-csv
  vel
  )

;;returns list of fallopian-tube-force-profiles
(define (make-fallopian-tube-forces-profile world increment)
  (if (> (length (world-probes world)) 1)
      (error "not implemented for more than one probe"))
  (let* ((topology (world-topology world))
	 (topo-L (topology-L topology)))
    (reverse
     (let loop ((alpha 0))
       (if (>= alpha topo-L)
	   '()
	   (let ((partial-result
		  (compute-fallopian-tube-forces-at-alpha world
							  alpha
							  (make-3point 0 0 0) ;pos-offset
							  )))
	     (cons partial-result
		   (loop (+ alpha increment)))
	     ))) ;;end of looop
     )))

;;used by both make-fallopian-tube-forces-profile and plot-simulation-results
(define (compute-fallopian-tube-forces-at-alpha world alpha
						#!optional time vel pos-offset)
  (if (default-object? pos-offset)
      (set! pos-offset (make-3point 0 0 0)))
  (let* ((topology (world-topology world))
	 (pos-f (topology-pos-f topology))
	 (probe (first (world-probes world)))
	 (mass (probe-mass probe))
	 (pos (3point-+ (pos-f alpha topology)
			pos-offset))
	 (state (vector-copy (world-initial-state world))))
    ;;force position entry in state to be alpha
    ;;set velocity to 0 - looking at static forces
    (vector-set! state (probe-position-reference probe) alpha)
    (vector-set! state (probe-velocity-reference probe) 0.)
    (let* ((tf-collection (compute-tf-collection world probe state pos-offset))
	   (Fprop-mag (torque-and-forces-collection-Fpropulsive-mag tf-collection))
	   (Ftranslational (torque-and-forces-collection-Ftranslational tf-collection))
	   (Ftranslational-mag (3point-magnitude Ftranslational))
	   (halflen (dipole-halflen (magnet-dipole (probe-magnet probe))))
	   (Torque (torque-and-forces-collection-Torque tf-collection))
	   (Frotational-mag (/ (3point-magnitude Torque) halflen))
	   (max-sidewall-force (/ (+ (abs Ftranslational-mag)
				     (abs Frotational-mag))
				  2)) ;;divide by 2 because force is spread over both endpoints of the probe
	   ;;***think about this division by 2 - should it only be on rotational or on both? ***
	   ;;***Depends on the usage of max-sidewall-force as a design criteria? ***
	   (aprop (/ Fprop-mag mass))
	   (asidewall (/ max-sidewall-force mass))
	   (atrans (/ Ftranslational-mag mass))
	   (arotational (/ Frotational-mag mass))
	   (partial-result
	    (make-fallopian-tube-force-profile alpha
					       Fprop-mag
					       pos
					       aprop
					       max-sidewall-force
					       asidewall
					       Torque
					       Ftranslational-mag
					       atrans
					       Ftranslational
					       Frotational-mag
					       arotational
					       time ;;for plot-simulation-results-csv
					       vel ;;alphadot - for plot-simulation-results-csv
					       )))
      partial-result)))

(define (plot-accel-profile world #!optional win)
  (smart-plot fallopian-tube-force-profile-alpha
	      fallopian-tube-force-profile-propulsive-accel
	      (make-fallopian-tube-forces-profile
	       world
	       *plot-fallopian-tube-default-increment*)
	      win))

;;; alpha force 3pt sidewallforce a asidewall
;;; 0 1 2 3 4 5
(define (plot-sidewall-accel-profile world #!optional win)
  (smart-plot fallopian-tube-force-profile-alpha
	      fallopian-tube-force-profile-max-sidewall-accel
	      (make-fallopian-tube-forces-profile
	       world
	       *plot-fallopian-tube-default-increment*)
	      win))

(define (mouse win)
  (let loop ()
    (let ((coord (get-pointer-coordinates win list)))
      (format #t "~%Frame coordinates are: (~a, ~a)"
	      (first coord)
	      (second coord))
      (if (not (= 0 (third coord))) ;;right button press?
	  win
	  (loop)))))

;;phase portrait shows position vs. velocity
(define (phase-portrait world #!optional win)
  (let* ((p (first (world-probes world)))
	 (p-ref (probe-position-reference p))
	 (v-ref (probe-velocity-reference p)))
    (smart-plot (lambda (s)
		  (vector-ref s p-ref))
		(lambda (s)
		  (vectore-ref s v-ref))
		(world-state-history world)
		win)))
