;;; The contents of this directory structure is subject to the following
;;; copyright notice:
;;;
;;;	Copyright (c) 2006, 2007 Andrew A. Berlin
;;;
;;;	This program is free software; you can redistribute it and/or
;;;	modify it under the terms of the GNU General Public License as
;;;	published by the Free Software Foundation; either version 2 of
;;;	the License, or (at your option) any later version.
;;;
;;;	This program is distributed in the hope that it will be
;;;	useful, but WITHOUT ANY WARRANTY; without even the implied
;;;	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
;;;	PURPOSE.  See the GNU General Public License for more details.
;;;
;;;	You should have received a copy of the GNU General Public
;;;	License along with this program; if not, write to the Free
;;;	Software Foundation, Inc., 59 Temple Place - Suite 330,
;;;	Boston, MA 02111-1307, USA.

(define dt 0.05)
(define eps 0.0001)
(define *run-until* 0.4) ;seconds



#|
Specific topological models:

Model #1: Harmonic-Linear-Harmonic (HLH)
For modelling fallopian tube, model as a function of length along the tube by a cubic:
As a function of length along the tube alpha, as alpha goes from 0 to L (where L is max length of tube):
let deltax = endpt-startpt
x = startpt.x + (deltax - deltax*cos(alpha * pi/2 / L)) Starts at right and heads left slowly at first then more sharply 
y = startpt.y + deltay * alpha / L
z = startpt.z + deltaz * sin(alpha * pi/2 / L) 

dx/dalpha = deltax*sin(alpha*pi/2*L)*pi/2/L
dy/dalpha = deltay*1/L
dz/dalpha = deltaz*cos(alpha*pi/2*L)*pi/2/L
then normalize to create a unit vector
|#

;;; x is lateral to patient (+ to right, - to left)
;;; y is lengthwise relative to patient (bellybutton is 0, head is +)
 ;; z is belly button (+) to back (-)
(define (pos-f-hlh alpha topology)
  (let* ((deltapt (topology-deltapt topology))
	 (startpt (topology-startpt topology))
	 (L (topology-L topology))
	 (x (+ (3point-x startpt)
	       (* (3point-x deltapt)
		  (- 1 (cos (* alpha pi/2 (/ L)))))))
	 (y (+ (3point-y startpt)
	       (* (3point-y deltapt)
		  (/ alpha L))))
	 (z (+ (3point-z startpt)
	       (* (3point-z deltapt)
		  (sin (* alpha pi/2 (/ L)))))))
    (make-3point x y z)))

(define (dir-f-hlh alpha topology)
  (let* ((deltapt (topology-deltapt topology))
	 (startpt (topology-startpt topology))
	 (L (topology-L topology))
	 (deltax (3point-x deltapt))
	 (deltay (3point-y deltapt))
	 (deltaz (3point-z deltapt))
	 (dx/dalpha (* deltax
		       (sin (* alpha pi/2 (/ L)))
		       (* pi/2 (/ L))))
	 (dy/dalpha (* deltay
		       (/ 1 L)))
	 (dz/dalpha (* deltaz
		       (cos (* alpha pi/2 (/ L)))
		       (* pi/2 (/ L)))))
    (make-unit-3point dx/dalpha
		       dy/dalpha
		       dz/dalpha)
    ))

(define (make-hlh-fallopian-tube startpt endpt L)
  (make-fallopian-tube 'hlh startpt endpt pos-f-hlh dir-f-hlh L))

;;model2: linear fallopian tube
(define (make-linear-model2-fallopian-tube startpt endpt L)
  (make-fallopian-tube 'linear_model2 startpt endpt pos-f-linear-model2 dir-f-linear-model2 L))

(define (pos-f-linear-model2 alpha topology)
  (3point-+ (topology-startpt topology)
	    (make-3point alpha 0 0)))

(define (dir-f-linear-model2 alpha topology)
  (make-unit-3point 1 0 0))


(define hlh-topology
  (make-hlh-fallopian-tube
   (make-3point (in->m -1)
		(in->m -1)
		(in->m -2))
   (make-3point (in->m -3)
		(in->m 1)
		(in->m 1))
   (in->m 8)))

(define linear-model2-start-point
  (make-3point 0
	       0
	       0))

(define linear-model2-end-point
   (make-3point (in->m 5)
		0
		0))

(define (linear-distance-between-2-points p1 p2)
  (let ((difference-pt (3point-- p1 p2)))
    (sqrt (3point-dot-product difference-pt difference-pt))))

(define linear-model2-topology
  (make-linear-model2-fallopian-tube
   linear-model2-start-point
   linear-model2-end-point
   (linear-distance-between-2-points
    linear-model2-end-point
    linear-model2-start-point)))

;;based on 1mmx4mm magnet
(define model1-probe-mass 2.26796e-5) ;;Kg
(define model1-probe-Br (gauss->tesla 12510.0)) ;;Tesla
(define model1-probe-length (mm->m 4.0))
(define model1-probe-diameter (mm->m 1.0))
(define model1-probe-initial-location 0.) ;;relative to start of fallopian tube
(define model1-probe-initial-velocity 0.) ;;relative to fallopian tube

;;based on 1mmx3mm magnet
(define model3-probe-mass 1.9844666e-5) ;;Kg
(define model3-probe-Br 1.421) ;;Tesla
(define model3-probe-length (mm->m 3.0))
(define model3-probe-diameter (mm->m 1.0))
(define model3-probe-initial-location 0.) ;;relative to start of fallopian tube
(define model3-probe-initial-velocity 0.) ;;relative to fallopian tube


;;;main probe is the variable that is used everywhere
;;set main probe to be model1, model3, or whatever other probe is to bs used.
(define main-probe-mass model3-probe-mass)
(define main-probe-Br model3-probe-Br)
(define main-probe-length model3-probe-length)
(define main-probe-diameter model3-probe-diameter)
(define main-probe-initial-location model3-probe-initial-location)
(define main-probe-initial-velocity model3-probe-initial-velocity)

(define (make-main-probe! world)
  (let* ((name 'main-probe)
	 (probe-initial-location main-probe-initial-location) ;;relative to start of fallopian tube
	 (probe-initial-velocity main-probe-initial-velocity) ;;relative to distance along fallopian tube
	 (mass main-probe-mass)
	 (len main-probe-length)
	 (diameter main-probe-diameter)
	 (Br main-probe-Br)
	 (magnet (make-cylindrical-magnet name
					  mass
					  len
					  diameter
					  Br)))
    (make-probe-and-add-to-world! world
				  name
				  probe-initial-location
				  probe-initial-velocity
				  magnet)
    ))

(define (make-actuator-identical-to-probe name position-3pt target-for-orientation #!optional invert-polarity-p)
  (if (default-object? invert-polarity-p)
      (set! invert-polarity-p false))
  (let* ((mass main-probe-mass)
	 (len main-probe-length)
	 (diameter main-probe-diameter)
	 (Br main-probe-Br)
	 (magnet (make-cylindrical-magnet name
					  mass
					  len
					  diameter
					  Br)))
    (make-custom-actuator name position-3pt target-for-orientation invert-polarity-p magnet)))




(define magnet-initial-orientation-target
  (pos-f-hlh 0 hlh-topology))

(define a0-on-axis (make-custom-actuator 'actuator0-on-axis
					   ;;linear-model2-end-point
					   (make-3point (in->m 10.0) 0 0)
					   linear-model2-start-point ;;target orientation
;;					   'invert-polarity
					   ))

(define a0-like-probe-magnet-separation-0.098g (cm->m 3.84693097))
(define a0-like-probe-magnet-separation-0.98g (cm->m 2.163306326))
(define a0-like-probe-magnet-separation-9.8g (cm->m 1.216516546))
;;(define a0-like-probe-magnet-separation-98g (cm->m 0.684097527)) ;;too close - div0

(define a0-like-probe-magnet-separation a0-like-probe-magnet-separation-0.098g)
;;(define a0-like-probe-magnet-separation a0-like-probe-magnet-separation-0.98g)
;;(define a0-like-probe-magnet-separation a0-like-probe-magnet-separation-9.8g)
;;(define a0-like-probe-magnet-separation a0-like-probe-magnet-separation-98g)


(define a0-like-probe-on-axis
  (make-actuator-identical-to-probe 'actuator0-identical-to-probe-on-axis
				    (make-3point a0-like-probe-magnet-separation 0 0)
				    linear-model2-start-point ;;target orientation
				    #f ;;invert-polarity
				    ))

(define a1-top-right (make-custom-actuator 'actuator1-top-right
					     (pt-above (in->m 2.0)
						       (pt-to-right (in->m 2.0)
								    (pt-towards-feet (in->m 2.0)
										     origin)))
					     magnet-initial-orientation-target))

(define a2-top-left (make-custom-actuator 'actuator1-top-left
					     (pt-above (in->m 2.0)
						       (pt-to-left (in->m 2.0)
								    (pt-towards-feet (in->m 2.0)
										     origin)))
					     magnet-initial-orientation-target))

(define a3-bottom-right (make-custom-actuator 'actuator1-bottom-right
					     (pt-below (in->m 2.0)
						       (pt-to-right (in->m 2.0)
								    (pt-towards-feet (in->m 2.0)
										     origin)))
					     magnet-initial-orientation-target))

(define a4-bottom-left (make-custom-actuator 'actuator1-bottom-left
					     (pt-below (in->m 2.0)
						       (pt-to-left (in->m 2.0)
								    (pt-towards-feet (in->m 2.0)
										     origin)))
					     magnet-initial-orientation-target))



(define world (make-new-world linear-model2-topology))

(define (initialize-world! world)
  (make-main-probe! world)
  (add-actuator-to-world! world a0-like-probe-on-axis)

;;  (add-actuator-to-world! world a0-on-axis)
;;  (add-actuator-to-world! world a1-top-right)
;;  (add-actuator-to-world! world a2-top-left)
;;  (add-actuator-to-world! world a3-bottom-right)
;;  (add-actuator-to-world! world a4-bottom-left)
  (set-world-latest-state! world (world-initial-state world))
  (set-world-state-history! world (list (world-initial-state world)))
;;add more here if needed
  )

(initialize-world! world)





