;;; The contents of this directory structure is subject to the following
;;; copyright notice:
;;;
;;;	Copyright (c) 2006, 2007 Andrew A. Berlin
;;;
;;;	This program is free software; you can redistribute it and/or
;;;	modify it under the terms of the GNU General Public License as
;;;	published by the Free Software Foundation; either version 2 of
;;;	the License, or (at your option) any later version.
;;;
;;;	This program is distributed in the hope that it will be
;;;	useful, but WITHOUT ANY WARRANTY; without even the implied
;;;	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
;;;	PURPOSE.  See the GNU General Public License for more details.
;;;
;;;	You should have received a copy of the GNU General Public
;;;	License along with this program; if not, write to the Free
;;;	Software Foundation, Inc., 59 Temple Place - Suite 330,
;;;	Boston, MA 02111-1307, USA.

(declare (usual-integrations))

(define *seek-error-tolerance-percentage* 0.1) ;;0.1% error tolerated
(define *seek-target-not-found* '(*seek-target-not-found*))
(define (seek-target-converged? returned-value)
  (not (eq? returned-value *seek-target-not-found*)))
(define (seek-target target f start end #!optional tolerance)
  (if (default-object? tolerance)
      (set! tolerance *seek-error-tolerance-percentage*))
  (let* ((opt (minimize
	       (lambda (x) (abs (- (f x) target)))
	       start end))
	 (x (car opt))
	 (fx (f x))
	 (delta (abs (- fx target)))
	 (err-percentage (* 100 (/ delta target))))
    (if (> err-percentage tolerance)
	*seek-target-not-found* ;;notify caller that do not have an answer
	opt) ;;return optimization result
    ))


;;Finds the point on the tube at which torque is closest to torque-target
(define (compute-torque-balance-pt world torque-target)
  (if (> (length (world-probes world)) 1)
      (error "compute torque balance not implemented for multiple probes"))
  (let ((L (topology-L (world-topology world)))) ;;length of tube
    (seek-target torque-target
		 (lambda (alpha) (compute-torque-mag world alpha))
		 0
		 L)))

(define (compute-torque-mag world alpha)
  (3point-magnitude
   (fallopian-tube-force-profile-Torque
    (compute-fallopian-tube-forces-at-alpha world
					    alpha
					    (make-3point 0 0 0) ;pos-offset
					    ))))

(define (report-torque-balance world torque-target)
  (let ((min-result (compute-torque-balance-pt world torque-target)))
    (if (seek-target-converged? min-result)
	(let* ((alpha (car min-result))
	       (val (compute-torque-mag world alpha)))
	  (format #t "~%Torque balance pt at alpha: ~a cm actual: ~a; target: ~a; delta: ~a%"
		  (* alpha 100)		;report in cm
		  val
		  torque-target
		  (* 100 (/ (abs (- val torque-target))
			    torque-target))))
	(format #t "~%Torque balance point NOT ACHIEVED for target: ~a"
		torque-target)
	)))

(define (report-sidewall-accel-balance world sidewall-target)
  (let* ((min-result (compute-sidewall-accel-balance-pt world sidewall-target)))
    (if (seek-target-converged? min-result)
	(let* ((alpha (car min-result))
	       (val (compute-sidewall-accel world alpha)))
	  (format #t "~%Sidewall accel balance pt at alpha: ~a cm actual: ~a; target: ~a; delta: ~a%"
		  (* alpha 100)		;report in cm
		  val
		  sidewall-target
		  (* 100 (/ (abs (- val sidewall-target))
			    sidewall-target))))
	(format #t "~%Sidewall balance point NOT ACHIEVED for target: ~a" sidewall-target)
	)))

(define (compute-sidewall-accel world alpha)
  (fallopian-tube-force-profile-max-sidewall-accel
   (compute-fallopian-tube-forces-at-alpha world
					   alpha
					   (make-3point 0 0 0) ;pos-offset
					   )))

(define (compute-sidewall-accel-balance-pt world sidewall-accel-target)
  (if (> (length (world-probes world)) 1)
      (error "compute sidewall balance not implemented for multiple probes"))
  (let* ((L (topology-L (world-topology world))) ;;length of tube
	 )
    (seek-target sidewall-accel-target
		 (lambda (alpha) (compute-sidewall-accel world alpha))
		 0
		 L)))

(define (report-prop-accel-balance world prop-target)
  (let ((min-result (compute-prop-accel-balance-pt world prop-target)))
    (if (seek-target-converged? min-result)
	(let* ((alpha (car min-result))
	       (val (compute-prop-accel world alpha)))
	  (format #t "~%Prop accel balance pt at alpha: ~a cm actual: ~a; target: ~a; delta: ~a%"
		  (* alpha 100)		;report in cm
		  val
		  prop-target
		  (* 100 (/ (abs (- val prop-target))
			    prop-target))))
	(format #t "~%Prop accel balance point NOT ACHIEVED for target: ~a" prop-target)
	)))

(define (compute-prop-accel world alpha)
  (fallopian-tube-force-profile-propulsive-accel
   (compute-fallopian-tube-forces-at-alpha world
					   alpha
					   (make-3point 0 0 0) ;pos-offset
					   )))

(define (compute-prop-accel-balance-pt world prop-accel-target)
  (if (> (length (world-probes world)) 1)
      (error "compute prop balance not implemented for multiple probes"))
  (let* ((p (first (world-probes world)))
	 (L (topology-L (world-topology world))) ;;length of tube
	 )
    (seek-target prop-accel-target 
		 (lambda (alpha) (compute-prop-accel world alpha))
		 0
		 L)))

(define (m->cm x)
  (* x 100))
